@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2021 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
 import copy
@@ -26,6 +26,7 @@ from typing import (
     List,
     NamedTuple,
     Optional,
+    Sequence,
     Tuple,
     Type,
     Union,
@@ -43,8 +44,6 @@ from .auth_webbrowser import AuthByWebBrowser
 from .bind_upload_agent import BindUploadError
 from .compat import IS_LINUX, IS_WINDOWS, quote, urlencode
 from .constants import (
-    DEFAULT_S3_CONNECTION_POOL_SIZE,
-    MAX_S3_CONNECTION_POOL_SIZE,
     PARAMETER_AUTOCOMMIT,
     PARAMETER_CLIENT_PREFETCH_THREADS,
     PARAMETER_CLIENT_REQUEST_MFA_TOKEN,
@@ -73,6 +72,7 @@ from .errorcode import (
     ER_CONNECTION_IS_CLOSED,
     ER_FAILED_PROCESSING_PYFORMAT,
     ER_FAILED_PROCESSING_QMARK,
+    ER_FAILED_TO_CONNECT_TO_DB,
     ER_INVALID_VALUE,
     ER_NO_ACCOUNT_NAME,
     ER_NO_NUMPY,
@@ -80,7 +80,7 @@ from .errorcode import (
     ER_NO_USER,
     ER_NOT_IMPLICITY_SNOWFLAKE_DATATYPE,
 )
-from .errors import DatabaseError, Error, ProgrammingError
+from .errors import DatabaseError, Error, OperationalError, ProgrammingError
 from .incident import IncidentAPI
 from .network import (
     DEFAULT_AUTHENTICATOR,
@@ -120,7 +120,7 @@ SUPPORTED_PARAMSTYLES = {
     "pyformat",
 }
 # Default configs, tuple of default variable and accepted types
-DEFAULT_CONFIGURATION = {
+DEFAULT_CONFIGURATION: Dict[str, Tuple[Any, Union[Type, Tuple[Type, ...]]]] = {
     "dsn": (None, (type(None), str)),  # standard
     "user": ("", str),  # standard
     "password": ("", str),  # standard
@@ -131,7 +131,7 @@ DEFAULT_CONFIGURATION = {
     "proxy_port": (None, (type(None), str)),  # snowflake
     "proxy_user": (None, (type(None), str)),  # snowflake
     "proxy_password": (None, (type(None), str)),  # snowflake
-    "protocol": (u"http", str),  # snowflake
+    "protocol": ("http", str),  # snowflake
     "warehouse": (None, (type(None), str)),  # snowflake
     "region": (None, (type(None), str)),  # snowflake
     "account": (None, (type(None), str)),  # snowflake
@@ -164,7 +164,6 @@ DEFAULT_CONFIGURATION = {
         (type(None), int),
     ),  # snowflake
     "client_prefetch_threads": (4, int),  # snowflake
-    "s3_connection_pool_s": (DEFAULT_S3_CONNECTION_POOL_SIZE, int),  # boto3 pool size
     "numpy": (False, bool),  # snowflake
     "ocsp_response_cache_filename": (None, (type(None), str)),  # snowflake internal
     "converter_class": (DefaultConverterClass(), SnowflakeConverter),
@@ -192,7 +191,6 @@ DEFAULT_CONFIGURATION = {
     ),  # only use regional url when the param is set
     # Allows cursors to be re-iterable
     "reuse_results": (False, bool),
-    "use_new_put_get": (True, bool),
 }
 
 APPLICATION_RE = re.compile(r"[\w\d_]+")
@@ -242,7 +240,6 @@ class SnowflakeConnection(object):
         network_timeout: Network timeout. Used for general purpose.
         client_session_keepalive: Whether to keep connection alive by issuing a heartbeat.
         client_session_keep_alive_heartbeat_frequency: Heartbeat frequency to keep connection alive in seconds.
-        s3_connection_pool_size: Size of connection pool for S3 boto driver.  Default is 10.
         client_prefetch_threads: Number of threads to download the result set.
         rest: Snowflake REST API object. Internal use only. Maybe removed in a later release.
         application: Application name to communicate with Snowflake as. By default, this is "PythonConnector".
@@ -392,15 +389,6 @@ class SnowflakeConnection(object):
         self._client_session_keep_alive_heartbeat_frequency = value
         self._validate_client_session_keep_alive_heartbeat_frequency()
 
-    @property
-    def _s3_connection_pool_size(self) -> int:
-        return self._s3_connection_pool_s
-
-    @_s3_connection_pool_size.setter
-    def _s3_connection_pool_size(self, value: int) -> None:
-        self._s3_connection_pool_s = value
-        self._validate_s3_connection_pool_size()
-
     @property
     def client_prefetch_threads(self):
         return (
@@ -493,7 +481,7 @@ class SnowflakeConnection(object):
         self._enable_stage_s3_privatelink_for_us_east_1 = True if value else False
 
     @arrow_number_to_decimal.setter
-    def arrow_number_to_decimal(self, value: bool):
+    def arrow_number_to_decimal_setter(self, value: bool):
         self._arrow_number_to_decimal = value
 
     def connect(self, **kwargs):
@@ -605,7 +593,7 @@ class SnowflakeConnection(object):
         remove_comments: bool = False,
         return_cursors: bool = True,
         cursor_class: SnowflakeCursor = SnowflakeCursor,
-        **kwargs
+        **kwargs,
     ) -> Iterable[SnowflakeCursor]:
         """Executes a SQL text including multiple statements. This is a non-standard convenience method."""
         stream = StringIO(sql_text)
@@ -620,7 +608,7 @@ class SnowflakeConnection(object):
         stream: StringIO,
         remove_comments: bool = False,
         cursor_class: SnowflakeCursor = SnowflakeCursor,
-        **kwargs
+        **kwargs,
     ) -> Generator["SnowflakeCursor", None, None]:
         """Executes a stream of SQL statements. This is a non-standard convenient method."""
         split_statements_list = split_statements(
@@ -1022,20 +1010,56 @@ class SnowflakeConnection(object):
         )
 
         auth = Auth(self.rest)
-        auth.authenticate(
-            auth_instance=auth_instance,
-            account=self.account,
-            user=self.user,
-            database=self.database,
-            schema=self.schema,
-            warehouse=self.warehouse,
-            role=self.role,
-            passcode=self._passcode,
-            passcode_in_password=self._passcode_in_password,
-            mfa_callback=self._mfa_callback,
-            password_callback=self._password_callback,
-            session_parameters=self._session_parameters,
-        )
+        try:
+            auth.authenticate(
+                auth_instance=auth_instance,
+                account=self.account,
+                user=self.user,
+                database=self.database,
+                schema=self.schema,
+                warehouse=self.warehouse,
+                role=self.role,
+                passcode=self._passcode,
+                passcode_in_password=self._passcode_in_password,
+                mfa_callback=self._mfa_callback,
+                password_callback=self._password_callback,
+                session_parameters=self._session_parameters,
+            )
+        except OperationalError:
+            logger.debug(
+                "Operational Error raised at authentication"
+                f"for authenticator: {type(auth_instance).__name__}"
+            )
+
+            while True:
+                try:
+                    auth_instance.handle_timeout(
+                        authenticator=self._authenticator,
+                        service_name=self.service_name,
+                        account=self.account,
+                        user=self.user,
+                        password=self._password,
+                    )
+                    auth.authenticate(
+                        auth_instance=auth_instance,
+                        account=self.account,
+                        user=self.user,
+                        database=self.database,
+                        schema=self.schema,
+                        warehouse=self.warehouse,
+                        role=self.role,
+                        passcode=self._passcode,
+                        passcode_in_password=self._passcode_in_password,
+                        mfa_callback=self._mfa_callback,
+                        password_callback=self._password_callback,
+                        session_parameters=self._session_parameters,
+                    )
+                except OperationalError as auth_op:
+                    if auth_op.errno == ER_FAILED_TO_CONNECT_TO_DB:
+                        raise auth_op
+                    logger.debug("Continuing authenticator specific timeout handling")
+                    continue
+                break
 
     def _write_params_to_byte_rows(
         self, params: List[Tuple[Union[Any, Tuple]]]
@@ -1101,20 +1125,13 @@ class SnowflakeConnection(object):
     # TODO we could probably rework this to not make dicts like this: {'1': 'value', '2': '13'}
     def _process_params_qmarks(
         self,
-        params: Union[List, Tuple, None],
+        params: Optional[Sequence],
         cursor: Optional["SnowflakeCursor"] = None,
-    ) -> Dict[str, Dict[str, str]]:
+    ) -> Optional[Dict[str, Dict[str, str]]]:
         if not params:
             return None
         processed_params = {}
 
-        if not isinstance(params, (list, tuple)):
-            errorvalue = {
-                "msg": "Binding parameters must be a list: {}".format(params),
-                "errno": ER_FAILED_PROCESSING_PYFORMAT,
-            }
-            Error.errorhandler_wrapper(self, cursor, ProgrammingError, errorvalue)
-
         get_type_and_binding = partial(self._get_snowflake_type_and_binding, cursor)
 
         for idx, v in enumerate(params):
@@ -1140,57 +1157,78 @@ class SnowflakeConnection(object):
                 logger.debug("idx: %s, type: %s", k, v.get("type"))
         return processed_params
 
-    def _process_params(
-        self, params: Union[Dict, List, Tuple, None], cursor: "SnowflakeCursor" = None
-    ) -> Union[Tuple, Dict]:
+    def _process_params_pyformat(
+        self,
+        params: Optional[Union[Any, Sequence[Any], Dict[Any, Any]]],
+        cursor: Optional["SnowflakeCursor"] = None,
+    ) -> Union[Tuple[Any], Dict[str, Any]]:
+        """Process parameters for client-side parameter binding.
+
+        Args:
+            params: Either a sequence, or a dictionary of parameters, if anything else
+                is given then it will be put into a list and processed that way.
+            cursor: The SnowflakeCursor used to report errors if necessary.
+        """
         if params is None:
             return {}
         if isinstance(params, dict):
-            return self.__process_params_dict(params)
+            return self._process_params_dict(params)
 
+        # TODO: remove this, callers should send in what's in the signature
         if not isinstance(params, (tuple, list)):
             params = [
                 params,
             ]
 
         try:
-            res = params
-            res = map(self.converter.to_snowflake, res)
-            res = map(self.converter.escape, res)
-            res = map(self.converter.quote, res)
+            res = map(self._process_single_param, params)
             ret = tuple(res)
-            logger.debug("parameters: %s", ret)
+            logger.debug(f"parameters: {ret}")
             return ret
         except Exception as e:
-            errorvalue = {
-                "msg": "Failed processing pyformat-parameters; {}".format(e),
-                "errno": ER_FAILED_PROCESSING_PYFORMAT,
-            }
-            Error.errorhandler_wrapper(self, cursor, ProgrammingError, errorvalue)
+            Error.errorhandler_wrapper(
+                self,
+                cursor,
+                ProgrammingError,
+                {
+                    "msg": f"Failed processing pyformat-parameters; {e}",
+                    "errno": ER_FAILED_PROCESSING_PYFORMAT,
+                },
+            )
 
-    def __process_params_dict(
-        self, params: Union[Dict, List, Tuple, None], cursor: "SnowflakeCursor" = None
+    def _process_params_dict(
+        self, params: Dict[Any, Any], cursor: Optional["SnowflakeCursor"] = None
     ) -> Dict:
-        # TODO this function could be reworked
         try:
-            to_snowflake = self.converter.to_snowflake
-            escape = self.converter.escape
-            quote = self.converter.quote
-            res = {}
-            for k, v in params.items():
-                c = v
-                c = to_snowflake(c)
-                c = escape(c)
-                c = quote(c)
-                res[k] = c
-            logger.debug("parameters: %s", res)
+            res = {k: self._process_single_param(v) for k, v in params.items()}
+            logger.debug(f"parameters: {res}")
             return res
         except Exception as e:
-            errorvalue = {
-                "msg": "Failed processing pyformat-parameters: {}".format(e),
-                "errno": ER_FAILED_PROCESSING_PYFORMAT,
-            }
-            Error.errorhandler_wrapper(self, cursor, ProgrammingError, errorvalue)
+            Error.errorhandler_wrapper(
+                self,
+                cursor,
+                ProgrammingError,
+                {
+                    "msg": f"Failed processing pyformat-parameters: {e}",
+                    "errno": ER_FAILED_PROCESSING_PYFORMAT,
+                },
+            )
+
+    def _process_single_param(self, param: Any) -> Any:
+        """Process a single parameter to Snowflake understandable form.
+
+        This is a convenience function to replace repeated multiple calls with a single
+        function call.
+
+        It calls the following underlying functions in this order:
+            1. self.converter.to_snowflake
+            2. self.converter.escape
+            3. self.converter.quote
+        """
+        to_snowflake = self.converter.to_snowflake
+        escape = self.converter.escape
+        _quote = self.converter.quote
+        return _quote(escape(to_snowflake(param)))
 
     def _cancel_query(self, sql, request_id):
         """Cancels the query with the exact SQL query and requestId."""
@@ -1259,13 +1297,6 @@ class SnowflakeConnection(object):
         )
         return self.client_session_keep_alive_heartbeat_frequency
 
-    def _validate_s3_connection_pool_size(self) -> None:
-        if self._s3_connection_pool_s <= 0:
-            self._s3_connection_pool_s = 1
-        elif self._s3_connection_pool_s > MAX_S3_CONNECTION_POOL_SIZE:
-            self._s3_connection_pool_s = MAX_S3_CONNECTION_POOL_SIZE
-        self._s3_connection_pool_s = int(self._s3_connection_pool_s)
-
     def _validate_client_prefetch_threads(self):
         if self.client_prefetch_threads <= 0:
             self._client_prefetch_threads = 1
@@ -1421,9 +1452,10 @@ class SnowflakeConnection(object):
         """Checks whether given status is currently running."""
         return status in (
             QueryStatus.RUNNING,
-            QueryStatus.RESUMING_WAREHOUSE,
             QueryStatus.QUEUED,
+            QueryStatus.RESUMING_WAREHOUSE,
             QueryStatus.QUEUED_REPARING_WAREHOUSE,
+            QueryStatus.BLOCKED,
             QueryStatus.NO_DATA,
         )
 
