@@ -143,8 +143,8 @@ class SnowflakeFileMeta:
     require_compress: bool = False
     dst_file_name: Optional[str] = None
     dst_file_size: int = -1
+    intermediate_stream: Optional[IO[bytes]] = None
     src_stream: Optional[IO[bytes]] = None
-    real_src_stream: Optional[IO[bytes]] = None
     # Specific to Downloads only
     local_location: Optional[str] = None
 
@@ -636,7 +636,7 @@ class SnowflakeFileTransferAgent:
                 meta,
                 self._stage_info,
                 4 * megabyte,
-                self._use_s3_regional_url,
+                use_s3_regional_url=self._use_s3_regional_url,
             )
         elif self._stage_location_type == AZURE_FS:
             return SnowflakeAzureRestClient(
@@ -644,7 +644,7 @@ class SnowflakeFileTransferAgent:
                 self._credentials,
                 AZURE_CHUNK_SIZE,
                 self._stage_info,
-                self._use_s3_regional_url,
+                use_s3_regional_url=self._use_s3_regional_url,
             )
         elif self._stage_location_type == S3_FS:
             return SnowflakeS3RestClient(
@@ -652,7 +652,7 @@ class SnowflakeFileTransferAgent:
                 self._credentials,
                 self._stage_info,
                 S3_CHUNK_SIZE,
-                self._use_s3_regional_url,
+                use_s3_regional_url=self._use_s3_regional_url,
             )
         elif self._stage_location_type == GCS_FS:
             return SnowflakeGCSRestClient(
@@ -661,7 +661,7 @@ class SnowflakeFileTransferAgent:
                 self._stage_info,
                 self._cursor._connection,
                 self._command,
-                self._use_s3_regional_url,
+                use_s3_regional_url=self._use_s3_regional_url,
             )
         raise Exception(f"{self._stage_location_type} is an unknown stage type")
 
@@ -970,7 +970,7 @@ class SnowflakeFileTransferAgent:
                     SnowflakeFileMeta(
                         name=os.path.basename(self._src_files[0]),
                         src_file_name=self._src_files[0],
-                        src_stream=self._source_from_stream,
+                        intermediate_stream=self._source_from_stream,
                         src_file_size=self._source_from_stream.seek(0, os.SEEK_END),
                         stage_location_type=self._stage_location_type,
                         encryption_material=self._encryption_material[0]
