@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2021 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
 import datetime
@@ -24,32 +24,44 @@ DEFAULT_BATCH_SIZE = 10
 DEFAULT_NUM_OF_RETRY_TO_TRIGGER_TELEMETRY = 10
 REQUEST_TIMEOUT = 3
 
-TelemetryAPI = namedtuple('TelemetryAPI', ['url', 'api_key'])
-TelemetryServer = namedtuple('TelemetryServer', ['name', 'url', 'api_key'])
-TelemetryEventBase = namedtuple('TelemetryEventBase', ['name', 'tags', 'urgent', 'value'])
+TelemetryAPI = namedtuple("TelemetryAPI", ["url", "api_key"])
+TelemetryServer = namedtuple("TelemetryServer", ["name", "url", "api_key"])
+TelemetryEventBase = namedtuple(
+    "TelemetryEventBase", ["name", "tags", "urgent", "value"]
+)
 
 
 class TelemetryAPIEndpoint(object):
     SFCTEST = TelemetryAPI(
         url="https://sfctest.client-telemetry.snowflakecomputing.com/enqueue",
-        api_key="rRNY3EPNsB4U89XYuqsZKa7TSxb9QVX93yNM4tS6"
+        api_key="rRNY3EPNsB4U89XYuqsZKa7TSxb9QVX93yNM4tS6",
     )
     SFCDEV = TelemetryAPI(
         url="https://sfcdev.client-telemetry.snowflakecomputing.com/enqueue",
-        api_key="kyTKLWpEZSaJnrzTZ63I96QXZHKsgfqbaGmAaIWf"
+        api_key="kyTKLWpEZSaJnrzTZ63I96QXZHKsgfqbaGmAaIWf",
     )
     PROD = TelemetryAPI(
         url="https://client-telemetry.snowflakecomputing.com/enqueue",
-        api_key="wLpEKqnLOW9tGNwTjab5N611YQApOb3t9xOnE1rX"
+        api_key="wLpEKqnLOW9tGNwTjab5N611YQApOb3t9xOnE1rX",
     )
 
 
 class TelemetryServerDeployments(object):
-    DEV = TelemetryServer("dev", TelemetryAPIEndpoint.SFCTEST.url, TelemetryAPIEndpoint.SFCTEST.api_key)
-    REG = TelemetryServer("reg", TelemetryAPIEndpoint.SFCTEST.url, TelemetryAPIEndpoint.SFCTEST.api_key)
-    QA1 = TelemetryServer("qa1", TelemetryAPIEndpoint.SFCDEV.url, TelemetryAPIEndpoint.SFCDEV.api_key)
-    PREPROD3 = TelemetryServer("preprod3", TelemetryAPIEndpoint.SFCDEV.url, TelemetryAPIEndpoint.SFCDEV.api_key)
-    PROD = TelemetryServer("prod", TelemetryAPIEndpoint.PROD.url, TelemetryAPIEndpoint.PROD.api_key)
+    DEV = TelemetryServer(
+        "dev", TelemetryAPIEndpoint.SFCTEST.url, TelemetryAPIEndpoint.SFCTEST.api_key
+    )
+    REG = TelemetryServer(
+        "reg", TelemetryAPIEndpoint.SFCTEST.url, TelemetryAPIEndpoint.SFCTEST.api_key
+    )
+    QA1 = TelemetryServer(
+        "qa1", TelemetryAPIEndpoint.SFCDEV.url, TelemetryAPIEndpoint.SFCDEV.api_key
+    )
+    PREPROD3 = TelemetryServer(
+        "preprod3", TelemetryAPIEndpoint.SFCDEV.url, TelemetryAPIEndpoint.SFCDEV.api_key
+    )
+    PROD = TelemetryServer(
+        "prod", TelemetryAPIEndpoint.PROD.url, TelemetryAPIEndpoint.PROD.api_key
+    )
 
 
 ENABLED_DEPLOYMENTS = (
@@ -57,7 +69,7 @@ ENABLED_DEPLOYMENTS = (
     TelemetryServerDeployments.REG.name,
     TelemetryServerDeployments.QA1.name,
     TelemetryServerDeployments.PREPROD3.name,
-    TelemetryServerDeployments.PROD.name
+    TelemetryServerDeployments.PROD.name,
 )
 
 
@@ -75,16 +87,18 @@ class TelemetryEvent(TelemetryEventBase):
     def to_dict(self):
         """Transform this event into a dictionary."""
         event = dict()
-        event['Name'] = self.name
-        event['Urgent'] = self.urgent
-        event['Value'] = self.value
-        event['Tags'] = self.generate_tags()
-        event.update({
-            'UUID': str(uuid.uuid4()),
-            'Created_On': datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S'),
-            'Type': self.get_type(),
-            'SchemaVersion': 1
-        })
+        event["Name"] = self.name
+        event["Urgent"] = self.urgent
+        event["Value"] = self.value
+        event["Tags"] = self.generate_tags()
+        event.update(
+            {
+                "UUID": str(uuid.uuid4()),
+                "Created_On": datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
+                "Type": self.get_type(),
+                "SchemaVersion": 1,
+            }
+        )
         return event
 
     def get_deployment(self):
@@ -92,8 +106,8 @@ class TelemetryEvent(TelemetryEventBase):
         tags = self.tags
         if tags:
             for tag in tags:
-                if tag.get('Name', None) == "deployment":
-                    return tag.get('Value')
+                if tag.get("Name", None) == "deployment":
+                    return tag.get("Value")
 
         return "Unknown"
 
@@ -108,28 +122,26 @@ class TelemetryEvent(TelemetryEventBase):
 
         telemetry = TelemetryService.get_instance()
         # Add telemetry service generated tags
-        tags['driver'] = CLIENT_NAME
-        tags['version'] = str(SNOWFLAKE_CONNECTOR_VERSION)
-        tags['telemetryServerDeployment'] = telemetry.deployment.name
-        tags['connectionString'] = telemetry.get_connection_string()
+        tags["driver"] = CLIENT_NAME
+        tags["version"] = str(SNOWFLAKE_CONNECTOR_VERSION)
+        tags["telemetryServerDeployment"] = telemetry.deployment.name
+        tags["connectionString"] = telemetry.get_connection_string()
         if telemetry.context and len(telemetry.context) > 0:
             for k, v in telemetry.context.items():
                 if v is not None:
-                    tags['ctx_' + str(k).lower()] = str(v)
+                    tags["ctx_" + str(k).lower()] = str(v)
 
         return tags
 
 
 class TelemetryLogEvent(TelemetryEvent):
-
     def get_type(self):
-        return 'Log'
+        return "Log"
 
 
 class TelemetryMetricEvent(TelemetryEvent):
-
     def get_type(self):
-        return 'Metric'
+        return "Metric"
 
 
 class TelemetryService(object):
@@ -151,7 +163,9 @@ class TelemetryService(object):
         self._enabled = True
         self._queue = Queue()
         self.batch_size = DEFAULT_BATCH_SIZE
-        self.num_of_retry_to_trigger_telemetry = DEFAULT_NUM_OF_RETRY_TO_TRIGGER_TELEMETRY
+        self.num_of_retry_to_trigger_telemetry = (
+            DEFAULT_NUM_OF_RETRY_TO_TRIGGER_TELEMETRY
+        )
         self.context = dict()
         self.connection_params = dict()
         self.deployment = TelemetryServerDeployments.PROD
@@ -233,13 +247,17 @@ class TelemetryService(object):
 
     def is_deployment_enabled(self):
         """Returns whether or not this deployment is enabled."""
-        return (self.deployment.name in ENABLED_DEPLOYMENTS)
+        return self.deployment.name in ENABLED_DEPLOYMENTS
 
     def get_connection_string(self):
         """Returns the URL used to connect to Snowflake."""
-        return self.connection_params.get('protocol', '') + '://' + \
-               self.connection_params.get('host', '') + ':' + \
-               str(self.connection_params.get('port', ''))
+        return (
+            self.connection_params.get("protocol", "")
+            + "://"
+            + self.connection_params.get("host", "")
+            + ":"
+            + str(self.connection_params.get("port", ""))
+        )
 
     def add(self, event):
         """Adds a telemetry event to the queue. If the event is urgent, upload all telemetry events right away."""
@@ -270,53 +288,68 @@ class TelemetryService(object):
         self.context = dict()
 
         for key, value in connection_params.items():
-            if "password" not in key and \
-                    "passcode" not in key and \
-                    "privateKey" not in key:
+            if (
+                "password" not in key
+                and "passcode" not in key
+                and "privateKey" not in key
+            ):
                 self.context[key] = value
 
     def configure_deployment(self, connection_params):
         """Determines which deployment we are sending Telemetry OOB messages to."""
         self.connection_params = connection_params
-        account = self.connection_params.get('account') if self.connection_params.get('account') else ''
-        host = self.connection_params.get('host') if self.connection_params.get('host') else ''
-        port = self.connection_params.get('port', None)
+        account = (
+            self.connection_params.get("account")
+            if self.connection_params.get("account")
+            else ""
+        )
+        host = (
+            self.connection_params.get("host")
+            if self.connection_params.get("host")
+            else ""
+        )
+        port = self.connection_params.get("port", None)
 
         # Set as PROD by default
         deployment = TelemetryServerDeployments.PROD
-        if 'reg' in host or 'local' in host:
+        if "reg" in host or "local" in host:
             deployment = TelemetryServerDeployments.REG
             if port == 8080:
                 deployment = TelemetryServerDeployments.DEV
-        elif 'qa1' in host or 'qa1' in account:
+        elif "qa1" in host or "qa1" in account:
             deployment = TelemetryServerDeployments.QA1
-        elif 'preprod3' in host:
+        elif "preprod3" in host:
             deployment = TelemetryServerDeployments.PREPROD3
 
         self.deployment = deployment
 
-    def log_ocsp_exception(self, event_type, telemetry_data, exception=None, stack_trace=None, tags=None, urgent=False):
+    def log_ocsp_exception(
+        self,
+        event_type,
+        telemetry_data,
+        exception=None,
+        stack_trace=None,
+        tags=None,
+        urgent=False,
+    ):
         """Logs an OCSP Exception and adds it to the queue to be uploaded."""
         if tags is None:
             tags = dict()
         try:
             if self.enabled:
-                event_name = 'OCSPException'
+                event_name = "OCSPException"
                 if exception is not None:
-                    telemetry_data['exceptionMessage'] = str(exception)
+                    telemetry_data["exceptionMessage"] = str(exception)
                 if stack_trace is not None:
-                    telemetry_data['exceptionStackTrace'] = stack_trace
+                    telemetry_data["exceptionStackTrace"] = stack_trace
 
                 if tags is None:
                     tags = dict()
 
-                tags['eventType'] = event_type
+                tags["eventType"] = event_type
 
                 log_event = TelemetryLogEvent(
-                    name=event_name,
-                    tags=tags,
-                    urgent=urgent,
-                    value=telemetry_data
+                    name=event_name, tags=tags, urgent=urgent, value=telemetry_data
                 )
 
                 self.add(log_event)
@@ -324,19 +357,21 @@ class TelemetryService(object):
             # Do nothing on exception, just log
             logger.debug("Failed to log OCSP exception", exc_info=True)
 
-    def log_http_request_error(self,
-                               event_name,
-                               url,
-                               method,
-                               sqlstate,
-                               errno,
-                               response=None,
-                               retry_timeout=None,
-                               retry_count=None,
-                               exception=None,
-                               stack_trace=None,
-                               tags=None,
-                               urgent=False):
+    def log_http_request_error(
+        self,
+        event_name,
+        url,
+        method,
+        sqlstate,
+        errno,
+        response=None,
+        retry_timeout=None,
+        retry_count=None,
+        exception=None,
+        stack_trace=None,
+        tags=None,
+        urgent=False,
+    ):
         """Logs an HTTP Request error and adds it to the queue to be uploaded."""
         if tags is None:
             tags = dict()
@@ -345,36 +380,33 @@ class TelemetryService(object):
                 telemetry_data = dict()
                 response_status_code = -1
                 # This mimics the output of HttpRequestBase.toString() from JBDC
-                telemetry_data['request'] = "{} {}".format(method, url)
-                telemetry_data['sqlState'] = sqlstate
-                telemetry_data['errorCode'] = errno
+                telemetry_data["request"] = "{} {}".format(method, url)
+                telemetry_data["sqlState"] = sqlstate
+                telemetry_data["errorCode"] = errno
                 if response:
-                    telemetry_data['response'] = response.json()
-                    telemetry_data['responseStatusLine'] = str(response.reason)
+                    telemetry_data["response"] = response.json()
+                    telemetry_data["responseStatusLine"] = str(response.reason)
                     if response.status_code:
                         response_status_code = str(response.status_code)
-                        telemetry_data['responseStatusCode'] = response_status_code
+                        telemetry_data["responseStatusCode"] = response_status_code
                 if retry_timeout:
-                    telemetry_data['retryTimeout'] = str(retry_timeout)
+                    telemetry_data["retryTimeout"] = str(retry_timeout)
                 if retry_count:
-                    telemetry_data['retryCount'] = str(retry_count)
+                    telemetry_data["retryCount"] = str(retry_count)
                 if exception:
-                    telemetry_data['exceptionMessage'] = str(exception)
+                    telemetry_data["exceptionMessage"] = str(exception)
                 if stack_trace:
-                    telemetry_data['exceptionStackTrace'] = stack_trace
+                    telemetry_data["exceptionStackTrace"] = stack_trace
 
                 if tags is None:
                     tags = dict()
 
-                tags['responseStatusCode'] = response_status_code
-                tags['sqlState'] = str(sqlstate)
-                tags['errorCode'] = errno
+                tags["responseStatusCode"] = response_status_code
+                tags["sqlState"] = str(sqlstate)
+                tags["errorCode"] = errno
 
                 log_event = TelemetryLogEvent(
-                    name=event_name,
-                    tags=tags,
-                    value=telemetry_data,
-                    urgent=urgent
+                    name=event_name, tags=tags, value=telemetry_data, urgent=urgent
                 )
 
                 self.add(log_event)
@@ -382,21 +414,20 @@ class TelemetryService(object):
             # Do nothing on exception, just log
             logger.debug("Failed to log HTTP request error", exc_info=True)
 
-    def log_general_exception(self,
-                              event_name: str,
-                              telemetry_data: dict,
-                              tags: Optional[dict] = None,
-                              urgent: Optional[bool] = False):
+    def log_general_exception(
+        self,
+        event_name: str,
+        telemetry_data: dict,
+        tags: Optional[dict] = None,
+        urgent: Optional[bool] = False,
+    ):
         """Sends any type of exception through OOB telemetry."""
         if tags is None:
             tags = dict()
         try:
             if self.enabled:
                 log_event = TelemetryLogEvent(
-                    name=event_name,
-                    tags=tags,
-                    value=telemetry_data,
-                    urgent=urgent
+                    name=event_name, tags=tags, value=telemetry_data, urgent=urgent
                 )
                 self.add(log_event)
         except Exception:
@@ -417,26 +448,39 @@ class TelemetryService(object):
             logger.debug("Sending OOB telemetry data. Payload: {}".format(payload))
             if ENABLE_TELEMETRY_LOG:
                 # This logger guarantees the payload won't be masked. Testing purpose.
-                rt_plain_logger.debug("OOB telemetry data being sent is {}".format(payload))
+                rt_plain_logger.debug(
+                    "OOB telemetry data being sent is {}".format(payload)
+                )
 
             with requests.Session() as session:
                 headers = {
-                    'Content-type': 'application/json',
-                    'x-api-key': self.deployment.api_key
+                    "Content-type": "application/json",
+                    "x-api-key": self.deployment.api_key,
                 }
                 response = session.post(
                     self.deployment.url,
                     data=payload,
                     headers=headers,
-                    timeout=REQUEST_TIMEOUT
+                    timeout=REQUEST_TIMEOUT,
                 )
-                if response.status_code == OK and json.loads(response.text).get('statusCode', 0) == OK:
-                    logger.debug("telemetry server request success: %d", response.status_code)
+                if (
+                    response.status_code == OK
+                    and json.loads(response.text).get("statusCode", 0) == OK
+                ):
+                    logger.debug(
+                        "telemetry server request success: %d", response.status_code
+                    )
                 else:
-                    logger.debug("telemetry server request error: %d", response.status_code)
+                    logger.debug(
+                        "telemetry server request error: %d", response.status_code
+                    )
                     success = False
         except Exception as e:
-            logger.debug("Telemetry request failed, Exception response: %s, exception: %s", response, str(e))
+            logger.debug(
+                "Telemetry request failed, Exception response: %s, exception: %s",
+                response,
+                str(e),
+            )
             success = False
         finally:
             logger.debug("Telemetry request success=%s", success)
@@ -450,7 +494,11 @@ class TelemetryService(object):
         try:
             payload = json.dumps(logs)
         except Exception:
-            logger.debug("Failed to generate a JSON dump from the passed in telemetry OOB events. String representation of logs: %s" % str(logs), exc_info=True)
+            logger.debug(
+                "Failed to generate a JSON dump from the passed in telemetry OOB events. String representation of logs: %s"
+                % str(logs),
+                exc_info=True,
+            )
             payload = None
         _, masked_text, _ = SecretDetector.mask_secrets(payload)
         return masked_text
